/*
 * Copyright (C) 2019 Alexander Mikhaylenko <exalm7659@gmail.com>
 *
 * SPDX-License-Identifier: LGPL-2.1+
 */

#include "config.h"
#include "hdy-navigation-direction.h"

/**
 * SECTION:hdy-navigation-direction
 * @short_description: Swipe navigation directions.
 * @title: HdyNavigationDirection
 * @See_also: #HdyDeck, #HdyLeaflet
 */

/**
 * HdyNavigationDirection:
 * @HDY_NAVIGATION_DIRECTION_BACK: Corresponds to start or top, depending on orientation and text direction
 * @HDY_NAVIGATION_DIRECTION_FORWARD: Corresponds to end or bottom, depending on orientation and text direction
 *
 * Represents direction of a swipe navigation gesture in #HdyDeck and
 * #HdyLeaflet.
 *
 * Since: 1.0
 */
